uniform extern float4x4 WorldViewProj : WORLDVIEWPROJECTION;
uniform extern float4x4 World : WORLD;
uniform extern float4x4 WorldIT : WORLDINVERSETRANSPOSE;
uniform extern float4x4 ViewI : VIEWINVERSE;
uniform extern float3 DirToLight;

struct VS_INPUT
{
    float4 position : POSITION;
    float3 normal : NORMAL;
    float2 uvCoord : TEXCOORD0;
}; // Semikolon nach struct: C-Stil, nicht wie Java/C#

struct VS_OUTPUT
{
    float4 position : POSITION;
    float3 normal : TEXCOORD0;
    float3 view : TEXCOORD1;
};

struct PS_OUTPUT
{
	float4 color : COLOR;
};

VS_OUTPUT myStupidVertexShader(VS_INPUT vertexIn)
{
    VS_OUTPUT vertexOut;
    vertexOut.position = mul(vertexIn.position, WorldViewProj);
    vertexOut.normal = normalize(mul(vertexIn.normal, WorldIT));
    float3 worldEyePos = ViewI[3].xyz;
    float3 worldVertPos = mul(vertexIn.position, World).xyz;
    vertexOut.view = worldEyePos - worldVertPos; // nicht normalisieren, sonst etwas falsche Interpolation
    return vertexOut;
}

PS_OUTPUT myStupidPixelShader(VS_OUTPUT pixelIn)
{
	PS_OUTPUT pixelOut;
	float3 normal = normalize(pixelIn.normal);
	float3 view = normalize(pixelIn.view);
	
	// Die HLSL-Funktion "lit" erledigt alles Folgende in einer Zeile.
	float3 halfvec = normalize(view + DirToLight);
	float specular = dot(normal, halfvec);
	if(specular < 0)
	{
		specular = 0; // eleganter ohne Verzweigung: specular = max(0, ...)
	}
	float diffuse = dot(normal, DirToLight);
	if(diffuse < 0)
	{
		diffuse = 0;
		specular = 0;
	}	
    pixelOut.color = diffuse*float4(1,0,0,1) + pow(specular, 100.0)*float4(1,1,1,1);
	return pixelOut;
}

technique phong
{
    pass P0
    {
        vertexShader = compile vs_2_0 myStupidVertexShader();
        pixelShader = compile ps_2_0 myStupidPixelShader();
    }
}